from django.db import models


class SectionSlugManager(models.Manager):
    def __init__(self, section_field="primary_section", slug_field="slug",
            *args, **kwargs):
        super(SectionSlugManager, self).__init__(*args, **kwargs)
        self.section_field = section_field
        self.slug_field = slug_field

    def get_by_slug(self, slug):
        if slug[-1] == "/":
            slug = slug[0:-1]
        if slug[0] == "/":
            slug = slug[1:]

        try:
            section_slug, content_slug = slug.rsplit("/", 1)
            section_slug += "/"
        except ValueError:
            raise self.model.DoesNotExist

        kwargs = {
                "%s__full_slug" % self.section_field: section_slug,
                self.slug_field: content_slug,
        }
        qs = self.model.objects.filter(**kwargs)
        if hasattr(qs, "select_subclasses"):
            qs = qs.select_subclasses()
        try:
            return qs[0]
        except IndexError:
            raise self.model.DoesNotExist



from armstrong.core.arm_sections.managers import SectionSlugManager
from armstrong.core.arm_sections.models import Section
from django.db import models
from model_utils.managers import InheritanceManager


class Common(models.Model):
    title = models.CharField(max_length=20)
    sections = models.ManyToManyField(Section)
    slug = models.SlugField()

    objects = InheritanceManager()
    with_section = SectionSlugManager(section_field="sections")


class Article(Common):
    summary = models.TextField(default="Default", blank=True)


class Photo(Common):
    url = models.URLField(default="http://localhost/", blank=True)


class SimpleCommon(models.Model):
    title = models.CharField(max_length=20)
    primary_section = models.ForeignKey(Section)
    slug = models.SlugField()

    objects = models.Manager()
    with_section = SectionSlugManager()


class SimpleArticle(SimpleCommon):
    summary = models.TextField(default="Default", blank=True)


class SimplePhoto(SimpleCommon):
    url = models.URLField(default="http://localhost/", blank=True)


class NonStandardField(models.Model):
    title = models.CharField(max_length=20)
    sections_by_another_name = models.ForeignKey(Section)
    slugs_by_another_name = models.SlugField()

    objects = models.Manager()
    with_section = SectionSlugManager(section_field="sections_by_another_name",
            slug_field="slugs_by_another_name")


class SectionForeignKeyCommon(models.Model):
    title = models.CharField(max_length=20)
    primary_section = models.ForeignKey(Section)
    slug = models.SlugField()

    objects = InheritanceManager()
    with_section = SectionSlugManager()


class SectionForeignKeyArticle(SectionForeignKeyCommon):
    summary = models.TextField(default="Default", blank=True)    


class ComplexCommon(models.Model):
    title = models.CharField(max_length=20)
    primary_section = models.ForeignKey(Section)
    related_sections = models.ManyToManyField(Section, related_name='relatedcomplexcommon_set')
    slug = models.SlugField()

    objects = InheritanceManager()
    with_section = SectionSlugManager()

class ComplexArticle(ComplexCommon):
    summary = models.TextField(default="Default", blank=True)


class CustomSection(Section):
    pass



from django.core.exceptions import ObjectDoesNotExist

from ._utils import *
from ..models import Section
from arm_sections_support.models import *

from armstrong.core.arm_sections.backends import ItemFilter
from armstrong.core.arm_sections.managers import SectionSlugManager
from model_utils.managers import InheritanceManager


class ManyToManyBackendTestCase(ArmSectionsTestCase):
    def setUp(self):
        super(ManyToManyBackendTestCase, self).setUp()

        self.sports = Section.objects.get(slug='sports')

        self.article = Article.objects.create(
                title="Test Article",
                slug='test_article',
            )
        self.article.sections = [self.sports]
        self.article2 = Article.objects.create(
                title="Second Article",
                slug='second_article',
            )
        self.article2.sections = [self.sports]

    def test_backend_with_articles(self):
        """
        Test fetching items for content with a many-to-many relationship to sections.
        """
        with override_settings(ARMSTRONG_SECTION_ITEM_MODEL='armstrong.core.arm_sections.tests.arm_sections_support.models.Common'):
            self.assert_(self.article in self.sports.items)
            self.assert_(self.article2 in self.sports.items)


class ForeignKeyBackendTestCase(ArmSectionsTestCase):
    def setUp(self):
        super(ForeignKeyBackendTestCase, self).setUp()

        self.sports = Section.objects.get(slug='sports')
        self.weather = Section.objects.get(slug='weather')

        self.foreign_key_article = SectionForeignKeyArticle.objects.create(
                title="Test Foreign Key Article",
                slug='test_foreign_key_article',
                primary_section=self.sports
            )
        self.foreign_key_article2 = SectionForeignKeyArticle.objects.create(
                title="Second Foreign Key Article",
                slug='second_foreign_key',
                primary_section=self.weather
            )

    def test_backend_with_foreign_key_articles(self):
        """
        Test fetching items for content with a foreign key relationship to sections.
        """
        with override_settings(ARMSTRONG_SECTION_ITEM_MODEL='armstrong.core.arm_sections.tests.arm_sections_support.models.SectionForeignKeyCommon'):
            self.assert_(self.foreign_key_article in self.sports.items)
            self.assert_(self.foreign_key_article2 in self.weather.items)


class ComplexBackendTestCase(ArmSectionsTestCase):
    def setUp(self):
        super(ComplexBackendTestCase, self).setUp()

        self.pro_sports = Section.objects.get(slug='pro')
        self.weather = Section.objects.get(slug='weather')

        self.complex_article = ComplexArticle.objects.create(
                title="Test Complex Article",
                slug='test_complex_article',
                primary_section=self.pro_sports
            )
        self.complex_article.related_sections = [self.weather]

    def test_backend_with_complex_articles(self):
        """
        Test fetching items for content with foreign key and many-to-many relationships to sections.
        """
        with override_settings(ARMSTRONG_SECTION_ITEM_MODEL='armstrong.core.arm_sections.tests.arm_sections_support.models.ComplexCommon'):
            self.assert_(self.complex_article in self.pro_sports.items)
            self.assert_(self.complex_article in self.weather.items)

class HierarchyBackendTestCase(ArmSectionsTestCase):
    def setUp(self):
        super(HierarchyBackendTestCase, self).setUp()

        self.sports = Section.objects.get(slug='sports')
        self.pro_sports = Section.objects.get(slug='pro')
        self.weather = Section.objects.get(slug='weather')

        self.complex_article = ComplexArticle.objects.create(
                title="Test Complex Article",
                slug='test_complex_article',
                primary_section=self.pro_sports
            )
        self.complex_article.related_sections = [self.weather]

    def test_backend_with_section_hierarchy(self):
        """
        Test fetching items for a parent section of the associated section.
        """
        with override_settings(ARMSTRONG_SECTION_ITEM_MODEL='armstrong.core.arm_sections.tests.arm_sections_support.models.ComplexCommon'):
            self.assert_(self.complex_article in self.sports.items)
            self.assert_(self.complex_article in self.pro_sports.items)

class ManagerTestCase(ArmSectionsTestCase):
    def setUp(self):
        super(ManagerTestCase, self).setUp()
        self.item_filter = ItemFilter()

    def test_default_manager(self):
        """
        Test ItemFilter.get_manager with the default manager.
        """
        self.assertIsA(self.item_filter.get_manager(ComplexCommon),
            InheritanceManager)

    def test_custom_manager(self):
        """
        Test ItemFilter.get_manager with a custom manager.
        """
        self.item_filter.manager_attr = 'with_section'
        self.assertIsA(self.item_filter.get_manager(ComplexCommon),
            SectionSlugManager)



from django.core import urlresolvers

from ._utils import ArmSectionsTestCase
from ..models import Section
from .arm_sections_support.models import CustomSection


class SectionsViewTestCase(ArmSectionsTestCase):
    def setUp(self):
        super(SectionsViewTestCase, self).setUp()
        self.sports = Section.objects.get(slug='sports')
        self.us_pro_sports = Section.objects.get(full_slug='sports/pro/us')
        self.custom_section = CustomSection.objects.create(
                    title='Custom',
                    slug='custom',
                    summary='Custom section',
                    parent=None,
                )

    def test_view_responds_to_full_slug(self):
        url = urlresolvers.reverse(
                'section_detail',
                kwargs={'full_slug': self.sports.full_slug})
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)

        url = urlresolvers.reverse(
                'section_detail',
                kwargs={'full_slug': self.us_pro_sports.full_slug})
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)

    def test_view_has_section_in_context(self):
        url = urlresolvers.reverse(
                'section_detail',
                kwargs={'full_slug': self.sports.full_slug})
        response = self.client.get(url)
        self.assertEqual(response.context['section'], self.sports)

    def test_view_with_custom_section_model(self):
        url = urlresolvers.reverse(
                'custom_section_detail',
                kwargs={'full_slug': self.custom_section.full_slug})
        response = self.client.get(url)
        base_section = Section.objects.get(id=self.custom_section.id)
        self.assertEqual(response.context['section'], self.custom_section)
        self.assertNotEqual(response.context['section'], base_section)

    def test_view_for_missing_section_raises_404(self):
        url = urlresolvers.reverse(
                'section_detail',
                kwargs={'full_slug': 'not-a-section/nope/'})
        response = self.client.get(url)
        self.assertEquals(response.status_code, 404)

    def test_feed_view(self):
        url = urlresolvers.reverse(
                'section_feed',
                kwargs={'full_slug': self.us_pro_sports.full_slug})
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)

    def test_feed_view_for_missing_section_raises_404(self):
        url = urlresolvers.reverse(
                'section_feed',
                kwargs={'full_slug': 'not-a-section/nope/'})
        response = self.client.get(url)
        self.assertEquals(response.status_code, 404)

    def test_view_with_custom_queryset(self):
        url = urlresolvers.reverse(
                'custom_section_queryset_detail',
                kwargs={'full_slug': self.custom_section.full_slug})
        response = self.client.get(url)
        base_section = Section.objects.get(id=self.custom_section.id)
        self.assertEqual(response.context['section'], self.custom_section)
        self.assertNotEqual(response.context['section'], base_section)



from armstrong.core.arm_sections.views import SimpleSectionView

from .models import CustomSection


class CustomSectionView(SimpleSectionView):
    def get(self, request, **kwargs):
        self.object = self.get_object(queryset=CustomSection.objects.all())
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)



from datetime import datetime
from django.core.files import File
from django.conf import settings

from armstrong.dev.tests.utils import ArmstrongTestCase
from armstrong.dev.tests.utils.backports import *
from armstrong.dev.tests.utils.concrete import *
from armstrong.dev.tests.utils.users import *

from ..models import Section

import fudge


class ArmSectionsTestCase(ArmstrongTestCase):
    def setUp(self):
        super(ArmSectionsTestCase, self).setUp()
        self.sections = []
        data = [
                ('Local', 'local', 'All about local', None),
                ('Sports', 'sports', 'All about sports', None),
                ('College', 'college', 'All about college sports', 1),
                ('Pro', 'pro', 'All about pro sports', 1),
                ('US', 'us', 'All about US sports', 3),
                ('Weather', 'weather', 'All about weather', None),
                ]
        for title, slug, summary, parent in data:
            if parent is not None:
                parent = self.sections[parent]
            self.sections.append(Section.objects.create(
                    title=title,
                    slug=slug,
                    summary=summary,
                    parent=parent,
                ))



from django.contrib import admin
from django.conf.urls.defaults import *

from armstrong.core.arm_sections.views import SimpleSectionView, SectionFeed

from .models import CustomSection
from .views import CustomSectionView

admin.autodiscover()


urlpatterns = patterns('',
    url(r'^admin/', include(admin.site.urls)),
    url(r'^sections/(?P<full_slug>([\w\-\_]+/)+)$',
        SimpleSectionView.as_view(template_name='test_sections.html'),
        name='section_detail'),
    url(r'^feeds/sections/(?P<full_slug>([\w\-\_]+/)+)$',
        SectionFeed(section_view='section_detail'),
        name='section_feed'),
    url(r'^custom-sections/(?P<full_slug>([\w\-\_]+/)+)$',
        SimpleSectionView.as_view(template_name='test_sections.html',
                                  model=CustomSection),
        name='custom_section_detail'),
    url(r'^custom-sections-queryset/(?P<full_slug>([\w\-\_]+/)+)$',
        CustomSectionView.as_view(template_name='test_sections.html'),
        name='custom_section_queryset_detail'),
)



from django.conf import settings
from django.db.models import Q


class ItemFilter(object):
    manager_attr = 'objects'

    def get_manager(self, model):
        """Return the desired manager for the item model."""
        return getattr(model, self.manager_attr)

    def get_section_relations(self, section):
        """Find every relationship between section and the item model."""
        all_rels = section._meta.get_all_related_objects() + \
                   section._meta.get_all_related_many_to_many_objects()
        model_rels = []
        for related in all_rels:
            found = "%s.%s" % (related.model.__module__,
                    related.model.__name__)
            if found == settings.ARMSTRONG_SECTION_ITEM_MODEL:
                model_rels.append(related)
        return model_rels

    def filter_objects_by_section(self, rels, section):
        """Build a queryset containing all objects in the section subtree."""
        subtree = section.get_descendants(include_self=True)
        kwargs_list = [{'%s__in' % rel.field.name: subtree} for rel in rels]
        q = Q(**kwargs_list[0])
        for kwargs in kwargs_list[1:]:
            q |= Q(**kwargs)
        return self.get_manager(rels[0].model).filter(q)

    def process_items(self, items):
        """
        Perform extra actions on the filtered items.

        Example: Further filtering the items in the section to meet a
        custom need.
        """
        if hasattr(items, 'select_subclasses'):
            items = items.select_subclasses()
        return items

    def __call__(self, section):
        relations = self.get_section_relations(section)
        items = self.filter_objects_by_section(relations, section)
        return self.process_items(items)


find_related_models = ItemFilter()



from django.core.urlresolvers import reverse
from django.views.generic import DetailView
from django.contrib.syndication.views import Feed
from django.shortcuts import get_object_or_404

from .models import Section


class SimpleSectionView(DetailView):
    context_object_name = 'section'
    model = Section

    def get_object(self, queryset=None):
        return self.get_section(queryset=queryset)

    def get_section(self, queryset=None):
        if queryset is None:
            queryset = self.get_queryset()
        return get_object_or_404(queryset, full_slug=self.kwargs['full_slug'])


class SectionFeed(Feed):
    def __init__(self, section_view, *args, **kwargs):
        self.section_view = section_view

    def get_object(self, request, full_slug):
        return Section.objects.get(full_slug=full_slug)

    def title(self, section):
        return section.title

    def link(self, section):
        return reverse(self.section_view,
                kwargs={'full_slug': section.full_slug})

    def description(self, section):
        return section.summary

    def items(self, section):
        return section.items



from django.core.exceptions import ObjectDoesNotExist

from ._utils import *
from ..models import Section
from arm_sections_support.models import *

class SectionSlugManagerTestCase(ArmSectionsTestCase):
    """
    Test the workings of the SectionSlugManager.

    We compare using the `title` attribute because the QuerySet result
    and the self.object have some differences with attributes like
    `_primary_section_cache` and `simplecommon_ptr_id`.

    """
    def setUp(self):
        super(SectionSlugManagerTestCase, self).setUp()

        self.article = Article.objects.create(
                title="Test Article",
                slug='test_article',
            )
        self.article.sections = [self.sections[1]]
        self.article2 = Article.objects.create(
                title="Second Article",
                slug='second_article',
            )
        self.article2.sections = [self.sections[1]]
        self.simple_article = SimpleArticle.objects.create(
                title="Test Simple Article",
                slug='test_simple_article',
                primary_section=self.sections[1]
            )
        self.simple_article2 = SimpleArticle.objects.create(
                title="Second Simple Article",
                slug='second_simple',
                primary_section=self.sections[2]
            )

    def test_get_by_slug_real(self):
        full_slug = "%s%s" % (self.sections[1].full_slug, self.simple_article.slug)
        self.assertEqual(SimpleArticle.with_section.get_by_slug(full_slug).title,
                         self.simple_article.title)

        full_slug = "%s%s" % (self.sections[2].full_slug, self.simple_article2.slug)
        self.assertEqual(SimpleArticle.with_section.get_by_slug(full_slug).title,
                         self.simple_article2.title)

        full_slug = "%s%s" % (self.sections[1].full_slug, self.article.slug)
        self.assertEqual(Article.with_section.get_by_slug(full_slug).title,
                         self.article.title)

        full_slug = "%s%s" % (self.sections[1].full_slug, self.article2.slug)
        self.assertEqual(Article.with_section.get_by_slug(full_slug).title,
                         self.article2.title)

    def test_get_by_slug_fake(self):
        full_slug = "%sfakearticle" % self.sections[1].full_slug
        with self.assertRaises(ObjectDoesNotExist):
            Article.with_section.get_by_slug(full_slug)
            SimpleArticle.with_section.get_by_slug(full_slug)

        full_slug = "fakesection/%s" % self.simple_article.slug
        with self.assertRaises(ObjectDoesNotExist):
            Article.with_section.get_by_slug(full_slug)
            SimpleArticle.with_section.get_by_slug(full_slug)

        full_slug = "fakesection/fakearticle"
        with self.assertRaises(ObjectDoesNotExist):
            Article.with_section.get_by_slug(full_slug)
            SimpleArticle.with_section.get_by_slug(full_slug)

    def test_get_by_slug_select_subclasses(self):
        # SimpleCommon does not use the InheritanceManager
        full_slug = "%s%s" % (self.sections[1].full_slug, self.simple_article.slug)
        self.assertNotEqual(type(SimpleCommon.with_section.get_by_slug(full_slug)),
                            type(self.simple_article))

        self.assertNotEqual(type(SimpleCommon.with_section.get_by_slug(full_slug)),
                            type(SimpleArticle.objects.get(pk=self.simple_article.pk)))

        self.assertEqual(type(SimpleCommon.with_section.get_by_slug(full_slug)),
                         type(SimpleCommon.objects.get(pk=self.simple_article.pk)))


        # Common has InheritanceManager
        full_slug = "%s%s" % (self.sections[1].full_slug, self.article.slug)
        self.assertEqual(type(Common.with_section.get_by_slug(full_slug)),
                         type(self.article))

        self.assertEqual(type(Common.with_section.get_by_slug(full_slug)),
                         type(Article.objects.get(pk=self.article.pk)))

    def test_get_by_slug_multiple(self):
        """
        There is no ordering on the Model or QuerySet,
        so we expect the first instance to be returned
        """
        dup_simple_article = SimpleArticle.objects.create(
                title="aaaa",
                slug='test_simple_article',
                primary_section=self.sections[1]
            )
        full_slug = "%s%s" % (self.sections[1].full_slug, self.simple_article.slug)
        self.assertEqual(SimpleArticle.with_section.get_by_slug(full_slug).title,
                         self.simple_article.title)

        dup_simple_article.delete()  # clean up

    def test_get_by_slug_malformed_slug(self):
        full_slug = "%s" % self.sections[1].full_slug
        with self.assertRaises(ObjectDoesNotExist):
            SimpleArticle.with_section.get_by_slug(full_slug)

        full_slug = "fakesection/"
        with self.assertRaises(ObjectDoesNotExist):
            SimpleArticle.with_section.get_by_slug(full_slug)

        full_slug = "fakesection"
        with self.assertRaises(ObjectDoesNotExist):
            SimpleArticle.with_section.get_by_slug(full_slug)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Section'
        db.create_table('arm_sections_section', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('summary', self.gf('django.db.models.fields.TextField')(default='', blank=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=50, db_index=True)),
            ('full_slug', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('parent', self.gf('mptt.fields.TreeForeignKey')(to=orm['arm_sections.Section'], null=True, blank=True)),
            ('lft', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('rght', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('tree_id', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('level', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
        ))
        db.send_create_signal('arm_sections', ['Section'])


    def backwards(self, orm):
        
        # Deleting model 'Section'
        db.delete_table('arm_sections_section')


    models = {
        'arm_sections.section': {
            'Meta': {'object_name': 'Section'},
            'full_slug': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'to': "orm['arm_sections.Section']", 'null': 'True', 'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50', 'db_index': 'True'}),
            'summary': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        }
    }

    complete_apps = ['arm_sections']
